/*
 * Copyright by ams AG
 * All rights are reserved.
 *
 * IMPORTANT - PLEASE READ CAREFULLY BEFORE COPYING, INSTALLING OR USING
 * THE SOFTWARE.
 *
 * THIS SOFTWARE IS PROVIDED FOR USE ONLY IN CONJUNCTION WITH AMS PRODUCTS.
 * USE OF THE SOFTWARE IN CONJUNCTION WITH NON-AMS-PRODUCTS IS EXPLICITLY
 * EXCLUDED.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <linux/input.h>
#include <linux/i2c.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/i2c/ams/ams_tsl2585.h>
#include <linux/i2c/ams/ams_tsl2585_reg.h>
#include "ams_i2c.h"
#include "qmath.h"
#include "ams_tsl2585_coeff.h"
#include "ams_tsl2585_als.h"
#include <linux/math64.h>

#define MOD_CLOCK  1389			//nano second
#ifdef CONFIG_QUALCOMM_AP
#include <linux/sensors.h>
static struct sensors_classdev als_sensors_cdev = {
    .name = "tsl2585-als",
    .vendor = "AMS",
    .enabled = 0,
    .sensors_enable = NULL,
};
#endif

struct coefficient{
    int32_t photopic;
    int32_t ir;
    int32_t dgf;
    int32_t q_fbits;
};

struct als_chip {
    struct feature_callbacks callbacks;
    struct input_dev *input;
    struct ams_chip *chip;
    uint16_t num_samples;
    uint8_t channel_disable;
    uint8_t channels;
    uint8_t persist;
    uint8_t valid;
    struct coefficient coefA;
    struct coefficient coefB;
    struct coefficient coefC;
    uint32_t deltaP;
    bool enabled;
    bool auto_gain;
    int32_t uv_coef1;
    int32_t uv_coef2;
    int32_t uv_coef3;
    int32_t uv_coef4;
    int32_t uv_dcoef1;
    int32_t uv_dcoef2;
    int32_t uv_dcoef3;
    int32_t uv_dcoef4;
    int32_t q_fracbits;

#ifdef CONFIG_QUALCOMM_AP
    struct sensors_classdev cdev;
#endif
};

static uint16_t mod_gain_tbl[14] ={ 1,
                                    1,
                                    2,
                                    4,
                                    8,
                                    16,
                                    32,
                                    64,
                                    128,
                                    256,
                                    512,
                                    1024,
                                    2048,
                                    4096 };

/* Q_fracbits is 2 for the following table */
static uint16_t q_mod_gain_tbl[14] ={ 2, /* 0.5 */
                                    4,
                                    8,
                                    16,
                                    32,
                                    64,
                                    128,
                                    256,
                                    512,
                                    1024,
                                    2048,
                                    4096,
                                    8192,
                                    16384 };


static struct als_chip *get_als_chip(struct ams_chip *chip)
{
    return container_of(chip->als, struct als_chip, callbacks);
}


static void set_als_thresh(struct ams_chip *chip, uint32_t low, uint32_t high)
{
    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_AILT0, (uint8_t)(low & 0x000000FF));
    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_AILT1, (uint8_t)(low & 0x0000FF00) >> 8);
    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_AILT2, (uint8_t)(low & 0x00FF0000) >> 8);
    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_AIHT0, (uint8_t)(high & 0x000000FF));
    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_AIHT1, (uint8_t)(high & 0x0000FF00) >> 8);
    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_AIHT2, (uint8_t)(high & 0x00FF0000) >> 8);
}

static void set_als_num_samples(struct ams_chip *chip, uint16_t num_samples)
{
    struct als_chip *als = get_als_chip(chip);

    als->num_samples = num_samples;

    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_ALS_NR_SAMPLES0, (uint8_t)((uint16_t)num_samples & 0x00FF));
    ams_i2c_write(chip->client, chip->shadow,
                  TSL2585_REG_ALS_NR_SAMPLES1, (uint8_t)(((uint16_t)num_samples & 0x0700) >> 8));
}

static void get_als_num_samples(struct ams_chip *chip)
{
    struct als_chip *als = get_als_chip(chip);

    ams_i2c_read(chip->client, TSL2585_REG_ALS_NR_SAMPLES0, &chip->shadow[TSL2585_REG_ALS_NR_SAMPLES0]);
    ams_i2c_read(chip->client, TSL2585_REG_ALS_NR_SAMPLES1, &chip->shadow[TSL2585_REG_ALS_NR_SAMPLES1]);
    als->num_samples = 0;
    als->num_samples = (uint16_t)(*(uint8_t *)&chip->shadow[TSL2585_REG_ALS_NR_SAMPLES1] & 0x03) << 8;
    als->num_samples |= (uint16_t)*(uint8_t *)&chip->shadow[TSL2585_REG_ALS_NR_SAMPLES0];
}

static void set_als_persistence(struct ams_chip *chip, uint8_t persistence)
{
    struct als_chip *als = get_als_chip(chip);

    ams_i2c_modify(chip->client, chip->shadow,
                   TSL2585_REG_CFG5, TSL2585_MASK_APERS, persistence);
    als->persist = persistence;
}

static void enable_als(struct ams_chip *chip, uint8_t enable)
{
    struct als_chip *als = get_als_chip(chip);

    if (enable) {
        ams_i2c_modify(chip->client, chip->shadow,
                       TSL2585_REG_ENABLE, 0x2, 0x2);
        ams_i2c_modify(chip->client, chip->shadow,
                       TSL2585_REG_INTENAB, TSL2585_MASK_AIEN, TSL2585_MASK_AIEN);

    } else {
        ams_i2c_modify(chip->client, chip->shadow,
                       TSL2585_REG_INTENAB, TSL2585_MASK_AIEN, 0x00);
        ams_i2c_modify(chip->client, chip->shadow,
                       TSL2585_REG_ENABLE, 0x02, 0x00);

    }
    als->enabled = enable;
}


void read_als_data(struct ams_chip *chip, struct als_channels *channels)
{
    /* Using the default values on TSL2585S the ALS data is 20-bit wide --
     * 16bit integration data + 4bit residuals. Since the ALS_DATA registers
     * are only 16bit wide the 4bit residuals are only visible if the
     * highest 4 bits of the "normal" integration bits are 0. In this case
     * the "normal" integration bits get shifted by 4 to the left and the
     * residual bits are shown in the 4 lowest bits. Additionally, the
     * ALS_DATA0_SCALED_STATUS bit and the ALS_DATA1_SCALED_STATUS bit
     * gets set to 1 to indicate that for a 20 bit wide result 4 zero
     * bits have to be added on the high side of the word. Otherwise the
     * 4 additional bits need to be added on the low side. That means
     * in the latter case the scaling has to be done on the host side
     * by shifting the result by 4 to the left to always get the same
     * 20bit range.
     *
     * Therefore, to have the right information to interpret the ALS_DATA
     * results correctly, ALS_STATUS needs to be read first and the ALS_DATA
     * registers immediately afterwards. It is recommended to do this in one
     * I2C block read including the ALS_STATUS2 and ALS_STATUS3 registers since the latter
     * contains the ALS_DATA0_GAIN_STATUS, ALS_DATA1_GAIN_STATUS, and ALS_DATA2_GAIN_STATUS
     * fields that are necessary for the lux equation as well. Doing it
     * this way it is guaranteed by design that all ALS_DATA results are
     * from the same integration cycle.
     *
     */
    int8_t   opt_8_reg = 0;
    uint16_t uv_data = 0;
    uint16_t uv_corrected_data = 0;

    ams_i2c_blk_read(chip->client, TSL2585_REG_ALS_STATUS,
                     &chip->shadow[TSL2585_REG_ALS_STATUS], 9);

    /* Photopic */
    uint16_t data = (chip->shadow[TSL2585_REG_DATAH0] << 8) | chip->shadow[TSL2585_REG_DATAL0];
    channels->photopic = data;
    if (!(chip->shadow[TSL2585_REG_ALS_STATUS] &
          TSL2585_MASK_ALS_DATA0_SCALED_STATUS)) {
        channels->photopic = channels->photopic << 4;
    }
    channels->photopic_gain = (*(uint16_t *)&chip->shadow[TSL2585_REG_ALS_STATUS2] & 0x0F);

    /* Ir */
    data = (chip->shadow[TSL2585_REG_DATAH1] << 8) | chip->shadow[TSL2585_REG_DATAL1];
    channels->ir = data;
    if (!(chip->shadow[TSL2585_REG_ALS_STATUS] &
          TSL2585_MASK_ALS_DATA1_SCALED_STATUS)) {
        channels->ir = channels->ir << 4;
    }
    channels->ir_gain = ((*(uint16_t *)&chip->shadow[TSL2585_REG_ALS_STATUS2] & 0xF0) >> 4);

    /* UV */
    data = (chip->shadow[TSL2585_REG_DATAH2] << 8) | chip->shadow[TSL2585_REG_DATAL2];
    channels->uv = data;
    if (!(chip->shadow[TSL2585_REG_ALS_STATUS] &
          TSL2585_MASK_ALS_DATA2_SCALED_STATUS)) {
        channels->uv = channels->uv << 4;
    }
    channels->uv_gain = (*(uint16_t *)&chip->shadow[TSL2585_REG_ALS_STATUS3] & 0x0F);

    /* Correction UV responsivity compensation with OTP register  TSL2585_REG_P2RAM_OTP_8 */
    /* formula UVcorrected_data= UVdata*(1+(i2cread(OTP_8-127)/100))*/
    uv_data = channels->uv;

    /* Read the TSL2585_REG_P2RAM_OTP_8 register*/
    ams_i2c_read(chip->client, TSL2585_REG_P2RAM_OTP_8, &opt_8_reg);
    
    /* Scale the register value to 100 for fix point calculation*/
    uv_corrected_data = (uv_data*(100+( (opt_8_reg * 100) - 12700)/100))/100;

    /* Save the corrected UV data*/
    channels->uv = uv_corrected_data;

    /* print the UV data*/
    dev_info(&chip->client->dev, "read_als_data -> photopic: %d\n", channels->photopic);
    dev_info(&chip->client->dev, "read_als_data -> ir_data : %d\n", channels->ir );
    dev_info(&chip->client->dev, "read_als_data -> uv_data : %d, uv_corrected_data: %d, opt_8_reg: %d\n", uv_data, uv_corrected_data, opt_8_reg);
    dev_info(&chip->client->dev, "read_als_data -> ***********************************************\n");

}

/* -----------------------------------------------------
 *     Residual Bits      |          Gain              |
 * =====================================================
 *          4             |       0.5x up to 128x      |
 * -----------------------------------------------------
 *          3             |           256x             |
 * -----------------------------------------------------
 *          2             |           512x             |
 * -----------------------------------------------------
 *          1             |          1024x             |
 * -----------------------------------------------------
 *          0             |          2048x             |
 * -----------------------------------------------------
 */
static uint32_t rbits_for_gain(uint32_t gain)
{
    uint32_t rbits = 0;

    if (gain <= 128)
    {
        rbits = 4;
    }
    else if (gain <= 256)
    {
        rbits = 3;
    }
    else if (gain <= 512)
    {
        rbits = 2;
    }
    else if (gain <= 1024)
    {
        rbits = 1;
    }

    return rbits;
}


static uint64_t get_atime_ns(struct ams_chip *chip, uint32_t gain)
{
    //uint32_t sample_time;
    //uint32_t atime;
    uint64_t als_integration_time;
    struct als_chip *als = get_als_chip(chip);
    uint8_t residual_enable;

    als_integration_time = (als->num_samples+1) * (chip->sample_time+1) * MOD_CLOCK;

    ams_i2c_read(chip->client, TSL2585_REG_MEAS_SEQR_RESIDUAL_0, &residual_enable);
    if((residual_enable & 0x11) == 0x11)
    {
        chip->use_residuals = true;
    }
    else
    {
        chip->use_residuals = false;
    }

    /* No residuals */
    if (chip->use_residuals == true)
    {
        als_integration_time -= MOD_CLOCK * (2 + 2 + (1 << rbits_for_gain(gain)));
    }

    return als_integration_time;
}

/*
 * The lux equation for TSL2585 with boot and diffuser
 *  n=1 coefficient -> cofA: -1.076, cofB: 1.329, DGF: 66.441
 *  n=2 coefficient -> cofA:  0.337, cofB: 0.337, DGF: 16.844
 *  If IR/PHO < 1.093 n=1, else n=2:
 *  Lux = DGFn*((CoefAn*Pho)+(CoefBn*IR))/(ATime*AGain);
 */
static int compute_lux(struct ams_chip *chip, struct als_channels *channels)
{
    /* struct als_chip *als = get_als_chip(chip); */
    int lux = 0;
    int lux_frac = 0;
    uint32_t again, q_again;
    uint32_t atime, q_atime;
    uint32_t photopic_data, q_photopic_data;
    uint32_t ir_data, q_ir_data;
    uint32_t ir_raw_data;
    int32_t q_cofA, q_cofB, q_DGF;
    int32_t q_fracbits = 0;
    uint64_t numerator = (1LL<<32)/1000000;
    uint64_t q_ir_pho = Q_IR_DIV_PHO; /* 1.042 */
    uint64_t q_ir_pho_ratio;
    int64_t q_var1,q_var2,q_var3,q_var4,q_var5,q_var6;

    if ((chip == NULL) || (channels == NULL))
        return -1;

    /* Get q fraction bits */
    q_fracbits    = Q_LUX_FRAC_BITS;

    /* Raw data  */
    photopic_data = channels->photopic;
    ir_data = channels->ir;
    ir_raw_data = channels->ir;

    /* photopic channel is 0 */
    if (photopic_data  == 0)
    {
        return 0;
    }

    dev_info(&chip->client->dev, "#########################################################\n");
    dev_info(&chip->client->dev, "compute_lux -> Raw data                     ---> photopic_data: %d, ir_data: %d\n", photopic_data, ir_data );

    /* Normalize data if gains are different */
    if (channels->photopic_gain != channels->ir_gain)
    {
        /* Normalize data */
      uint32_t q_photopic_gain = q_mod_gain_tbl[channels->photopic_gain];
      uint32_t q_ir_gain       = q_mod_gain_tbl[channels->ir_gain];
      uint64_t q_gain_ratio    =  _qkdiv(q_ir_gain, q_photopic_gain, Q_MOD_GAIN_TBL_FRAC_BITS, Q_MOD_GAIN_TBL_FRAC_BITS, Q_MOD_GAIN_TBL_FRAC_BITS);
      q_ir_data                = _int2q(ir_data, q_fracbits);
      uint64_t q_ir_temp_data  = _qkdiv(q_ir_data, q_gain_ratio, q_fracbits, Q_MOD_GAIN_TBL_FRAC_BITS, q_fracbits);
      q_ir_data                = q_ir_temp_data;
    } else {
      q_ir_data       = _int2q(ir_data, q_fracbits);
    }

    /* Convert channel data to q_format */
    q_photopic_data = _int2q(photopic_data, q_fracbits);
    q_again         = q_mod_gain_tbl[channels->photopic_gain];

    /* IR/PHO < 1.042 find the ir/pho ratio */
    q_ir_pho_ratio = _qkdiv(q_ir_data, q_photopic_data, q_fracbits, q_fracbits, q_fracbits);
    dev_info(&chip->client->dev, "compute_lux -> ir_pho ratio                 ---> q_ir_pho_ratio: %lld, q_ir_pho: %lld\n", q_ir_pho_ratio, q_ir_pho );
    /* Get lux coef */
    if (q_ir_pho_ratio  < q_ir_pho)
    {
        q_cofA = Q_COEFA_PHOTOPIC;
        q_cofB = Q_COEFA_IR;
        q_DGF  = Q_COEFA_DGF;
	dev_info(&chip->client->dev, "compute_lux -> n=1 coefficient              ---> q_cofA: %d, q_cofB: %d, q_DGF: %d \n", q_cofA, q_cofB, q_DGF );
    }
    else
    {
        q_cofA = Q_COEFC_PHOTOPIC;
        q_cofB = Q_COEFC_IR;
        q_DGF  = Q_COEFC_DGF;
	dev_info(&chip->client->dev, "compute_lux -> n=2 coefficient              ---> q_cofA: %d, q_cofB: %d, q_DGF: %d \n", q_cofA, q_cofB, q_DGF );
    }

    again = mod_gain_tbl[channels->photopic_gain];
    /* Get atime */
    atime = get_atime_ns(chip, again);
    /* convert to milisecond */
    atime = ( atime * numerator ) >> 32;

    dev_info(&chip->client->dev, "compute_lux -> atime in milisecond          ---> atime: %d\n", atime);
    dev_info(&chip->client->dev, "compute_lux -> again                        ---> again: %d\n", again);
    if (atime == 0){
      atime = 1;
    }
    q_atime          = _int2q(atime, q_fracbits);

    /* Compute Lux */

    /* Lux = DGFn*((CoefAn*PHO)+(CoefBn*IR))/(ATime*AGain) */
    q_var1 = _qmul(q_cofA, q_photopic_data, q_fracbits, q_fracbits, q_fracbits);
    q_var2 = _qmul(q_cofB, q_ir_data, q_fracbits, q_fracbits, q_fracbits);
    q_var3 = qadd(q_var1, q_var2);
    q_var4 = _qmul(q_DGF, q_var3, q_fracbits, q_fracbits, q_fracbits);
    q_var5 = _qmul(q_atime, q_again, q_fracbits, Q_MOD_GAIN_TBL_FRAC_BITS, q_fracbits);
    q_var6 = _qkdiv(q_var4, q_var5, q_fracbits, q_fracbits, q_fracbits);
    lux = _q2int(q_var6, q_fracbits);
    lux_frac  = q_var6 & q_fracbits ;
    if (lux < 0)
    {
      dev_info(&chip->client->dev, "compute_lux -> lux with boot,diffuser       ---> zero or negative lux: %d \n", lux );
      return 0;
    }
    dev_info(&chip->client->dev, "compute_lux -> lux with boot,diffuser       ---> lux: %d, remainder: %d\n", lux, lux_frac);
    dev_info(&chip->client->dev, "#########################################################\n");
    return lux;
}

/*
    ALT_OVERRIDE = True is the default setting, and means to use only the typical equation
                   (works for typical indoor lighting conditions and outdoor conditions).
                   Setting this to False may give better results if testing indoors with special UV bulbs
*/
static const bool ALT_OVERRIDE = true;

/*
 * The uv index equation for TSL2585
 */
static int compute_uvindex(struct ams_chip *chip, struct als_channels *channels)
{
    int uvindex, uvc = 0;
    int uvindex_frac = 0;
    uint32_t photopic_data, q_photopic_data;
    uint32_t ir_data, q_ir_data;
    uint32_t uv_data, q_uv_data;
    int32_t q_uv_coef1, q_uv_coef2, q_uv_coef3, q_uv_coef4;
    int32_t ph_gain, ir_gain, uv_gain;
    uint32_t ph_atime, ir_atime, uv_atime;
    uint32_t gc_data;
    uint64_t numerator = (1LL<<32)/1000000;
    int64_t q_var1,q_var2,q_var3,q_var4,q_var5,q_var6;
    int32_t q_fracbits = 0;

    if ((chip == NULL) || (channels == NULL))
        return -1;

    /* Raw data  */
    q_fracbits    = Q_UV_FRAC_BITS;
    photopic_data = channels->photopic;
    ir_data       = channels->ir;
    uv_data       = channels->uv;
    gc_data       = 1;
    /* Convert channel data to q_format */
    q_photopic_data = _int2q(photopic_data, q_fracbits);
    q_ir_data       = _int2q(ir_data, q_fracbits);
    q_uv_data       = _int2q(uv_data, q_fracbits);

    dev_info(&chip->client->dev, "*********************************************************\n");
    dev_info(&chip->client->dev, "*********************************************************\n");
    dev_info(&chip->client->dev, "compute_uvindex -> Raw data                     ---> photopic_data: %d, ir_data: %d, uv_data: %d\n", photopic_data, ir_data, uv_data );

    uint32_t q_200         = _int2q(200, q_fracbits);
    uint32_t q_2           = _int2q(2, q_fracbits);
    uint32_t q_gc          = _int2q(gc_data, q_fracbits);
    dev_info(&chip->client->dev, "compute_uvindex -> q values                     ---> q_200: %d q_2: %d q_gc: %d\n",q_200, q_2, q_gc);

    /* Norm data = (raw_data * (200ms/ATIME) * GC_data) / (2*Gain) */
    /* Normalize photopic data */
    ph_atime               = get_atime_ns(chip, ph_gain);
    ph_atime               = ( ph_atime * numerator ) >> 32; /* to milisecond */
    dev_info(&chip->client->dev, "compute_uvindex -> atime in milisecond          ---> ph_atime: %d\n", ph_atime);
    uint32_t q_ph_gain     = q_mod_gain_tbl[channels->photopic_gain];
    uint32_t q_ph_atime    = _int2q(ph_atime, q_fracbits);
    uint64_t q_atime_ratio = _qkdiv(q_200, q_ph_atime, q_fracbits, q_fracbits, q_fracbits);
    uint32_t q_temp_data   = _qmul(q_photopic_data, q_atime_ratio, q_fracbits, q_fracbits, q_fracbits);
    uint32_t q_temp1_data  = _qmul(q_temp_data, q_gc, q_fracbits, q_fracbits, q_fracbits);
    uint32_t q_temp2_data  = _qmul(q_2, q_ph_gain, q_fracbits, q_fracbits, q_fracbits);
    q_photopic_data        = _qkdiv(q_temp1_data, q_temp2_data, q_fracbits, q_fracbits, q_fracbits);
    dev_info(&chip->client->dev, "compute_uvindex -> Normalize phototic data      ---> q_photopic_data: %d\n", q_photopic_data);

    /* Normalize ir data */
    ir_atime               = get_atime_ns(chip, ir_gain);
    ir_atime               = ( ir_atime * numerator ) >> 32; /* to milisecond */
    dev_info(&chip->client->dev, "compute_uvindex -> atime in milisecond          ---> ir_atime: %d\n", ir_atime);
    uint32_t q_ir_gain     = q_mod_gain_tbl[channels->ir_gain];
    uint32_t q_ir_atime    = _int2q(ir_atime, q_fracbits);
    q_atime_ratio          = _qkdiv(q_200, q_ir_atime, q_fracbits, q_fracbits, q_fracbits);
    q_temp_data            = _qmul(q_ir_data, q_atime_ratio, q_fracbits, q_fracbits, q_fracbits);
    q_temp1_data           = _qmul(q_temp_data, q_gc, q_fracbits, q_fracbits, q_fracbits);
    q_temp2_data           = _qmul(q_2, q_ir_gain, q_fracbits, q_fracbits, q_fracbits);
    q_ir_data              = _qkdiv(q_temp1_data, q_temp2_data, q_fracbits, q_fracbits, q_fracbits);
    dev_info(&chip->client->dev, "compute_uvindex -> Normalize ir       data      ---> q_ir_data: %d\n", q_ir_data);

    /* Normalize uv data */
    uv_atime               = get_atime_ns(chip, uv_gain);
    uv_atime               = ( uv_atime * numerator ) >> 32; /* to milisecond */
    dev_info(&chip->client->dev, "compute_uvindex -> atime in milisecond          ---> uv_atime: %d\n", uv_atime);
    uint32_t q_uv_gain     = q_mod_gain_tbl[channels->uv_gain];
    uint32_t q_uv_atime    = _int2q(uv_atime, q_fracbits);
    q_atime_ratio          = _qkdiv(q_200, q_uv_atime, q_fracbits, q_fracbits, q_fracbits);
    q_temp_data            = _qmul(q_uv_data, q_atime_ratio, q_fracbits, q_fracbits, q_fracbits);
    q_temp1_data           = _qmul(q_temp_data, q_gc, q_fracbits, q_fracbits, q_fracbits);
    q_temp2_data           = _qmul(q_2, q_uv_gain, q_fracbits, q_fracbits, q_fracbits);
    q_uv_data              = _qkdiv(q_temp1_data, q_temp2_data, q_fracbits, q_fracbits, q_fracbits);
    dev_info(&chip->client->dev, "compute_uvindex -> Normalize uv       data      ---> q_uv_data: %d\n", q_uv_data);

    if ( ALT_OVERRIDE == false ){
      /*  */
      q_uv_coef1 = Q_UV_DCOEF1;
      q_uv_coef2 = Q_UV_DCOEF2;
      q_uv_coef3 = Q_UV_DCOEF3;
      q_uv_coef4 = Q_UV_DCOEF4;
      dev_info(&chip->client->dev, "compute_uvindex -> ALT_OVERRIDE==False\n");
      dev_info(&chip->client->dev, "compute_uvindex -> q_uv_coef1: %d, q_uv_coef2: %d, q_uv_coef3: %d, q_uv_coef4: %d, q_fracbits: %d \n", q_uv_coef1, q_uv_coef2, q_uv_coef3, q_uv_coef4, q_fracbits );
    } else {
      q_uv_coef1 = Q_UV_COEF1;
      q_uv_coef2 = Q_UV_COEF2;
      q_uv_coef3 = Q_UV_COEF3;
      q_uv_coef4 = Q_UV_COEF4;
      dev_info(&chip->client->dev, "compute_uvindex -> ALT==False and ALT_OVERRIDE==True\n");
      dev_info(&chip->client->dev, "compute_uvindex -> q_uv_coef1: %d, q_uv_coef2: %d, q_uv_coef3: %d, q_uv_coef4: %d, q_fracbits: %d \n", q_uv_coef1, q_uv_coef2, q_uv_coef3, q_uv_coef4, q_fracbits );
    }

    /* Convert q_format back for printing */
    photopic_data    = _q2int(q_photopic_data, q_fracbits);
    ir_data          = _q2int(q_ir_data, q_fracbits);
    uv_data          = _q2int(q_uv_data, q_fracbits);
    dev_info(&chip->client->dev, "compute_uvindex -> Normalize data               ---> photopic_data: %d, ir_data: %d, uv_data: %d\n", photopic_data, ir_data, uv_data );

    /* UVI equation version 2.0 */
    /* UVC = MAX(UVA - C1 * IR - C2 * Y, 0) */
    /* C1*IR */
    q_var1 = _qmul(q_uv_coef1, q_ir_data, q_fracbits, q_fracbits, q_fracbits);

    /* C2*Y */
    q_var2 = _qmul(q_uv_coef2, q_photopic_data, q_fracbits, q_fracbits, q_fracbits);

    /* UVC = UVA - q_var1 - q_var2*/
    q_var3 = qsub(q_uv_data, q_var1);
    q_var4 = qsub(q_var3, q_var2);
    uvc    = _q2int(q_var4, q_fracbits);
    if (uvc < 0){
      uvc    = 0;
      q_var4 = 0;
    }

    /* UVI = MAX(C3 * UVC - C4, 0)*/
    q_var5 = _qmul(q_uv_coef3, q_var4, q_fracbits, q_fracbits, q_fracbits);
    q_var6 = qsub(q_var5, q_uv_coef4);
    uvindex = _q2int(q_var6, q_fracbits);
    if (uvindex < 0){
      uvindex = 0;
    }

    dev_info(&chip->client->dev, "compute_uvindex -> uvindex  ---> uvindex: %d.%d\n", uvindex, uvindex_frac );
    dev_info(&chip->client->dev, "*********************************************************\n");
    dev_info(&chip->client->dev, "\n");
    return uvindex;
}

static ssize_t enable_store(struct device *dev, struct device_attribute *attr,
                            const char *buf, size_t count)
{
    struct ams_chip *chip = dev_get_drvdata(dev);
    ssize_t ret;
    uint8_t enable;

    ret = kstrtol(buf, 0, (long *)(&(enable)));
    if (ret != 0)
        dev_err(&chip->client->dev, "kstrtol() error.\n");
    enable_als(chip, enable);
    return count;
}

static ssize_t enable_show(struct device *dev, struct device_attribute *attr,
                           char *buf)
{
    struct ams_chip *chip = dev_get_drvdata(dev);
    struct als_chip *als = get_als_chip(chip);
    int ret;

    AMS_MUTEX_LOCK(&chip->lock);
    ret = snprintf(buf, PAGE_SIZE, "%d\n", als->enabled);
    AMS_MUTEX_UNLOCK(&chip->lock);

    return ret;
}

static ssize_t data_show(struct device *dev, struct device_attribute *attr,
                         char *buf);
#define ALS_DATA_ATTR(name)                     \
    DEVICE_ATTR(name, 0644, data_show, NULL)

static ALS_DATA_ATTR(ch0);
static ALS_DATA_ATTR(ch1);
static ALS_DATA_ATTR(ch2);
static ALS_DATA_ATTR(valid);
static ALS_DATA_ATTR(raw);

static ssize_t coef_show(struct device *dev, struct device_attribute *attr,
                         char *buf);

#define ALS_COEF_ATTR(name)                     \
    DEVICE_ATTR(name, 0644, coef_show, NULL)

static ALS_COEF_ATTR(photopic);
static ALS_COEF_ATTR(ir);
static ALS_COEF_ATTR(dgf);
static ALS_COEF_ATTR(q_fbits);

static ssize_t config_show(struct device *dev, struct device_attribute *attr,
                           char *buf);
static ssize_t config_store(struct device *dev, struct device_attribute *attr,
                            const char *buf, size_t count);
#define ALS_CFG_ATTR(name)                              \
    DEVICE_ATTR(name, 0644, config_show, config_store)

static ALS_CFG_ATTR(num_samples);
static ALS_CFG_ATTR(persist);
static ALS_CFG_ATTR(channels);

static DEVICE_ATTR_RW(enable);

static struct attribute *config_attrs[] = {
    &dev_attr_num_samples.attr,
    &dev_attr_persist.attr,
    &dev_attr_channels.attr,
    &dev_attr_enable.attr,
    NULL,
};

enum als_config {
    NUM_SAMPLES_CFG=0,
    PERSIST_CFG,
    CHANNELS_CFG
};

static ssize_t config_show(struct device *dev, struct device_attribute *attr,
                           char *buf)
{
    struct ams_chip *chip = dev_get_drvdata(dev);
    struct als_chip *als = get_als_chip(chip);
    int ret = 0;
    uint32_t i;

    for (i = 0; i < (sizeof(config_attrs) / sizeof(struct attribute *) - 1); ++i) {
        if (!strncmp(attr->attr.name, config_attrs[i]->name, strlen(attr->attr.name))) {
            AMS_MUTEX_LOCK(&chip->lock);
            switch (i) {
            case NUM_SAMPLES_CFG:
                get_als_num_samples(chip);
                ret = snprintf(buf, PAGE_SIZE, "%d\n", als->num_samples);
                break;
            case PERSIST_CFG:
                ret = snprintf(buf, PAGE_SIZE, "%d\n", als->persist);
                break;
            case CHANNELS_CFG:
                ret = snprintf(buf, PAGE_SIZE, "%d\n", als->channels);
                break;
            default:
                ret = snprintf(buf, PAGE_SIZE, "not found\n");
                break;
            }
            AMS_MUTEX_UNLOCK(&chip->lock);
            return ret;
        }
    }
    return ret;
}

static ssize_t config_store(struct device *dev, struct device_attribute *attr,
                            const char *buf, size_t count)
{
    struct ams_chip *chip = dev_get_drvdata(dev);
    struct als_chip *als = get_als_chip(chip);
    uint32_t i = 0;
    uint32_t data;

    if (!kstrtol(buf, 0, (long *)(&(data)))) {
        for (i = 0; i < (sizeof(config_attrs) / sizeof(struct attribute *) - 1); ++i) {
            if (!strncmp(attr->attr.name, config_attrs[i]->name, strlen(attr->attr.name))) {
                AMS_MUTEX_LOCK(&chip->lock);
                switch (i) {
                case NUM_SAMPLES_CFG:
                    set_als_num_samples(chip, data);
                    break;
                case PERSIST_CFG:
                    set_als_persistence(chip, (uint8_t )data);
                    break;
                case CHANNELS_CFG:
                    als->channels = data;
                    break;
                default:
                    break;
                }
                AMS_MUTEX_UNLOCK(&chip->lock);
                return count;
            }
        }
    }
    return count;
}

static struct attribute *data_attrs[] = {
    &dev_attr_ch0.attr,
    &dev_attr_ch1.attr,
    &dev_attr_ch2.attr,
    &dev_attr_valid.attr,
    &dev_attr_raw.attr,
    NULL,
};

static ssize_t data_show(struct device *dev, struct device_attribute *attr,
                         char *buf)
{
    struct ams_chip *chip = dev_get_drvdata(dev);
    struct als_chip *als = get_als_chip(chip);
    int ret = 0;
    uint32_t i;

    for (i = 0; i < (sizeof(data_attrs) / sizeof(struct attribute *) - 1); ++i) {
        if (!strncmp(attr->attr.name, data_attrs[i]->name, strlen(attr->attr.name))) {
            AMS_MUTEX_LOCK(&chip->lock);
            if (i == 3) { //show valid
                ret = snprintf(buf, PAGE_SIZE, "%d\n", chip->channel_data.als_valid);
            }
            if (i == 4) { //show all raw data
                uint32_t ch0, ch1, ch2, pho_gain, ir_gain, uv_gain;
                ch0 = chip->channel_data.photopic;
                ch1 = chip->channel_data.ir;
                ch2 = chip->channel_data.uv;
		pho_gain = chip->channel_data.photopic_gain;
                ir_gain = chip->channel_data.ir_gain;
                uv_gain =  chip->channel_data.uv_gain;
                ret = snprintf(buf, PAGE_SIZE, "%d %d %d %d %d %d\n", ch0,pho_gain,ch1,ir_gain,ch2,uv_gain);
                AMS_MUTEX_UNLOCK(&chip->lock);
                return ret;
            }
            AMS_MUTEX_UNLOCK(&chip->lock);
            return ret;
        }
    }
    return ret;
}

static struct attribute *coef_attrs[] = {
    &dev_attr_photopic.attr,
    &dev_attr_ir.attr,
    &dev_attr_dgf.attr,
    &dev_attr_q_fbits.attr,
    NULL,
};

enum ams_coefs {
    COEF_PHOTOPIC=0,
    COEF_IR,
    COEF_DGF,
    COEF_Q_FBITS,
};

static ssize_t coef_show(struct device *dev, struct device_attribute *attr,
                         char *buf)
{
    struct ams_chip *chip = dev_get_drvdata(dev);
    struct als_chip *als = get_als_chip(chip);
    int ret = 0;
    uint32_t i;

    for (i = 0; i < (sizeof(coef_attrs) / sizeof(struct attribute *) - 1); ++i) {
        if (!strncmp(attr->attr.name, coef_attrs[i]->name, strlen(attr->attr.name))) {
            AMS_MUTEX_LOCK(&chip->lock);
            switch (i) {
            case COEF_PHOTOPIC:
                ret = snprintf(buf, PAGE_SIZE, "%d, %d, %d\n", als->coefA.photopic, als->coefB.photopic, als->coefC.photopic);
                break;
            case COEF_IR:
                ret = snprintf(buf, PAGE_SIZE, "%d, %d, %d\n", als->coefA.ir, als->coefB.ir, als->coefC.ir);
                break;
            case COEF_DGF:
                ret = snprintf(buf, PAGE_SIZE, "%d, %d, %d\n", als->coefA.dgf, als->coefB.dgf, als->coefC.dgf);
                break;
            case COEF_Q_FBITS:
                ret = snprintf(buf, PAGE_SIZE, "%d, %d, %d\n", als->coefA.q_fbits, als->coefB.q_fbits, als->coefC.q_fbits);
                break;
            default:
                ret = snprintf(buf, PAGE_SIZE, "not found\n");
                break;
            }
            AMS_MUTEX_UNLOCK(&chip->lock);
            return ret;

        }
    }
    return ret;
}


static const struct attribute_group config_group = {
    .name = "config",
    .attrs = config_attrs,
};

static const struct attribute_group data_group = {
    .name = "data",
    .attrs = data_attrs,
};

static const struct attribute_group coef_group = {
    .name = "coefficients",
    .attrs = coef_attrs,
};

static const struct attribute_group *als_groups[] = {
    &config_group,
    &data_group,
    &coef_group,
    NULL,
};

static void report_als(struct ams_chip *chip, uint64_t value)
{
    struct als_chip *als = get_als_chip(chip);

    if (als->input) {
        input_report_abs(als->input, ABS_MISC, value);
        input_sync(als->input);
    }
}

static void als_irq(struct ams_chip *chip, uint8_t status)
{
    uint8_t als_status = 0;
    struct als_chip *als = get_als_chip(chip);

    if (status & TSL2585_MASK_AINT) {
        ams_i2c_read(chip->client, TSL2585_REG_STATUS2,
                     &chip->shadow[TSL2585_REG_STATUS2]);
        als_status = chip->shadow[TSL2585_REG_STATUS2];
        if (status & TSL2585_MASK_ASAT) {
            if (als_status & 0x10) { //Als saturation
                report_als(chip, (als_status & 0x10) * -1);
		chip->channel_data.als_valid = 0;
                return;
            }
        }
        if (als_status & 0x40) {    //ALS Valid
            read_als_data(chip, &chip->channel_data);
            chip->channel_data.als_valid = 1;
        } else {
            dev_info(&chip->client->dev, "ALS not valid %x\n", als_status);
            read_als_data(chip, &chip->channel_data);
            report_als(chip, -1); //Invalid
        }
        als->valid = 0;
    }
}

static void config_als(struct ams_chip *chip)
{
    struct als_chip *als = get_als_chip(chip);

    set_als_thresh(chip, 0x000000, 0xFFFFFF);
    set_als_persistence(chip, als->persist);
    set_als_num_samples(chip, als->num_samples);
}

static int als_open(struct input_dev *input)
{
    struct ams_chip *chip = dev_get_drvdata(&input->dev);

    dev_info(&chip->client->dev, "opening ALS\n");
    config_als(chip);
    enable_als(chip, 1);
    return 0;
}

static void als_close(struct input_dev *input)
{
    struct ams_chip *chip = dev_get_drvdata(&input->dev);

    ams_i2c_modify(chip->client, chip->shadow,
                   TSL2585_REG_INTENAB, TSL2585_MASK_AIEN, 0);
    dev_info(&chip->client->dev, "closing ALS\n");
    enable_als(chip, 0);
}

#ifdef CONFIG_QUALCOMM_AP
static int als_set_enable(struct sensors_classdev *sensors_cdev, unsigned int enable)
{
    struct als_chip *als = container_of(sensors_cdev, struct als_chip, cdev);

    if (enable) {
        als->input->open(als->input);
    } else {
        als->input->close(als->input);
    }

    return 0;
}
#endif

static int init_als_input(struct ams_chip *chip)
{
    struct als_chip *als = get_als_chip(chip);

    als->input = devm_input_allocate_device(&chip->client->dev);
    if (!als->input) {
        return -ENODEV;
    }
    als->input->name = "als";
    als->input->id.bustype = BUS_I2C;
    set_bit(EV_ABS, als->input->evbit);
    set_bit(ABS_MISC, als->input->absbit);
    input_set_abs_params(als->input, ABS_MISC, 0, 65535, 0, 0);
    als->input->open = als_open;
    als->input->close = als_close;
    input_set_drvdata(als->input, chip);
    if (input_register_device(als->input)) {
        dev_err(&chip->client->dev, "%s: Can't register input device '%s'\n", __func__, als->input->name);
        return -ENODEV;
    }
    if (sysfs_create_groups(&als->input->dev.kobj, als_groups)) {
        dev_err(&chip->client->dev, "Error creating sysfs attribute group.\n");
    }
#ifdef CONFIG_QUALCOMM_AP
    als->cdev = als_sensors_cdev;
    als->cdev.sensors_enable = als_set_enable;
    if (sensors_classdev_register(&als->input->dev, &als->cdev)) {
        dev_err(&chip->client->dev, "sensors class register failed.\n");
        return -ENODEV;
    }
#endif
    return 0;
}

#define READ_ALS_PROPERTY(name)                         \
    if (!of_property_read_s32(node, #name, &value)) {   \
        als->name = value;                              \
    } else {                                            \
        als->name = 0;                                  \
    }

#define READ_ALS_COEFA(name)                                    \
    if (!of_property_read_s32(node_coefA, #name, &value)) {     \
        als->coefA.name = value;                                \
    } else {                                                    \
        als->coefA.name = 0;                                    \
    }

#define READ_ALS_COEFB(name)                                    \
    if (!of_property_read_s32(node_coefB, #name, &value)) {     \
        als->coefB.name = value;                                \
    } else {                                                    \
        als->coefB.name = 0;                                    \
    }
#define READ_ALS_COEFC(name)                                    \
    if (!of_property_read_s32(node_coefC, #name, &value)) {     \
        als->coefC.name = value;                                \
    } else {                                                    \
        als->coefC.name = 0;                                    \
    }

#define READ_UV_PROPERTY(name)                         \
    if (!of_property_read_s32(node, #name, &value)) {   \
        als->name = value;                              \
    } else {                                            \
        als->name = 0;                                  \
    }

static int init_device_tree(struct ams_chip *chip)
{
    struct device_node *node  = of_get_child_by_name(chip->platform_data->of_node, "als");
    struct als_chip *als = get_als_chip(chip);
    int value = 0;
    struct device_node *node_coefA, *node_coefB, *node_coefC;

    if (node) { // Configuration
        READ_ALS_PROPERTY(num_samples);
        READ_ALS_PROPERTY(persist);
        READ_ALS_PROPERTY(deltaP);
        READ_ALS_PROPERTY(channels);
        node_coefA = of_get_child_by_name(node, "coefficientA");
        node_coefB = of_get_child_by_name(node, "coefficientB");
        node_coefC = of_get_child_by_name(node, "coefficientC");
        if (node_coefA) { // CoefficientA
            READ_ALS_COEFA(photopic);
            READ_ALS_COEFA(ir);
            READ_ALS_COEFA(dgf);
            READ_ALS_COEFA(q_fbits);
        }
        if (node_coefB)
        {
            READ_ALS_COEFB(photopic);
            READ_ALS_COEFB(ir);
            READ_ALS_COEFB(dgf);
            READ_ALS_COEFB(q_fbits);
        }
        if (node_coefC)
        {
            READ_ALS_COEFC(photopic);
            READ_ALS_COEFC(ir);
            READ_ALS_COEFC(dgf);
            READ_ALS_COEFC(q_fbits);
        }

        return 1;
    }
    return 0;
}

static int init_uv_device_tree(struct ams_chip *chip)
{
    struct device_node *node = of_get_child_by_name(chip->platform_data->of_node, "uv");
    struct als_chip *als = get_als_chip(chip);
    int value = 0;

    if (node) {
      READ_UV_PROPERTY(uv_coef1);
      READ_UV_PROPERTY(uv_coef2);
      READ_UV_PROPERTY(uv_coef3);
      READ_UV_PROPERTY(uv_coef4);
      READ_UV_PROPERTY(uv_dcoef1);
      READ_UV_PROPERTY(uv_dcoef2);
      READ_UV_PROPERTY(uv_dcoef3);
      READ_UV_PROPERTY(uv_dcoef4);
      READ_UV_PROPERTY(q_fracbits);

      return 1;
    }
    return 0;
}

void ams_tsl2585_remove_als(struct ams_chip *chip)
{
    struct als_chip *als = get_als_chip(chip);

    if (als->input) {
        sysfs_remove_link(&chip->input->dev.kobj, "als");
        sysfs_remove_groups(&als->input->dev.kobj, als_groups);
        input_unregister_device(als->input);
    }
}

int ams_tsl2585_init_als(struct ams_chip *chip)
{
    static struct als_chip als = { {0} };

    if (chip->input) {
        als.chip = chip;
        chip->als = &als.callbacks;
        if (init_device_tree(chip) && init_uv_device_tree(chip)) {
            als.callbacks.irq = als_irq;
            if (init_als_input(chip)) {
                return -ENODEV;
            }
            if (sysfs_create_link(&chip->input->dev.kobj, &als.input->dev.kobj, "als")) {
                //do nothing
            }
        }
        return 0;
    }
    return -ENODEV;
}

/*
 * The open air lux equation for TSL2585
 */
int compute_lux_open_air(struct ams_chip *chip, struct als_channels *channels)
{
    struct als_chip *als = get_als_chip(chip);
    uint64_t lux = 0;
    uint32_t again;
    uint32_t photopic_data;
    uint32_t ir_data;
    uint32_t ir_raw_data;
    int64_t ir_pho_ratio;
    int cofA, cofB, DGF;
    uint64_t cpl;
    uint64_t atime;

    if ((chip == NULL) || (channels == NULL))
        return -1;

    photopic_data = channels->photopic;
    ir_data = channels->ir;
    ir_raw_data = channels->ir;

    /* Normalize data if gains are different */
    if (channels->photopic_gain != channels->ir_gain)
    {
        /* Normalize data */
        int photopic_gain = mod_gain_tbl[channels->photopic_gain];
        int ir_gain = mod_gain_tbl[channels->ir_gain];
        int gain_ratio = ir_gain / photopic_gain;

        ir_data = ir_data / gain_ratio;

    }
    again = mod_gain_tbl[channels->photopic_gain];

    /* Get lux coef */
    /* photopic channel is 0 */
    if (photopic_data  == 0)
    {
        return 0;
    }
    /* IR/PHO < 1.981 find the ir/pho ratio */
    ir_pho_ratio = ((ir_data * 1000)  / photopic_data);
    if (ir_pho_ratio  < 1981)
    {
        cofA = als->coefA.photopic;
        cofB = als->coefA.ir;
        DGF = als->coefA.dgf;
	dev_info(&chip->client->dev, "n=1 coefficient -> cofA: %d, cofB: %d, DGF: %d \n", cofA, cofB, DGF );
    }
    else if ((ir_pho_ratio  >= 1981) && (ir_pho_ratio < 4404))
    {
        cofA = als->coefB.photopic;
        cofB = als->coefB.ir;
        DGF = als->coefB.dgf;
	dev_info(&chip->client->dev, "n=2 coefficient -> cofA: %d, cofB: %d, DGF: %d \n", cofA, cofB, DGF );
    }
    else
    {
        cofA = als->coefC.photopic;
        cofB = als->coefC.ir;
        DGF = als->coefC.dgf;
	dev_info(&chip->client->dev, "n=3 coefficient -> cofA: %d, cofB: %d, DGF: %d \n", cofA, cofB, DGF );
    }

    /* Get atime */
    atime = get_atime_ns(chip, again);
    /* convert to milisecond */
    do_div(atime, 1000000);
    if (atime == 0){
      atime = 1;
    }
    cpl = atime;

    /* Compute Lux */
    cpl = cpl * again;
    cpl = cpl * 1000;
    lux = ((photopic_data * cofA) + (ir_data * cofB));
    lux = lux * DGF;
    if (lux > 0)
    {
        do_div (lux, cpl);
        do_div(lux, 1000);
    }
    else
    {
        lux = 0;
    }

    dev_info(&chip->client->dev, "#########################################################\n");
    dev_info(&chip->client->dev, "compute_lux -> photopic_data: %d, ir_raw_data: %d, ir_data: %d, ir_pho_ratio: %lld, again: %d, atime: %lld, cpl: %lld, lux: %lld\n", photopic_data, ir_raw_data, ir_data, ir_pho_ratio, again, atime, cpl, lux);
    return lux;
}
