/*
*****************************************************************************
* Copyright by ams AG                                                       *
* All rights are reserved.                                                  *
*                                                                           *
* IMPORTANT - PLEASE READ CAREFULLY BEFORE COPYING, INSTALLING OR USING     *
* THE SOFTWARE.                                                             *
*                                                                           *
* THIS SOFTWARE IS PROVIDED FOR USE ONLY IN CONJUNCTION WITH AMS PRODUCTS.  *
* USE OF THE SOFTWARE IN CONJUNCTION WITH NON-AMS-PRODUCTS IS EXPLICITLY    *
* EXCLUDED.                                                                 *
*                                                                           *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  *
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     *
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     *
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     *
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE     *
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.      *
*****************************************************************************
*/

/*! \file
* \brief Device driver for monitoring ambient light intensity in (lux) and flicker
* with the AMS TSL2585.
*/

#ifndef TSL2585_REG_H
#define TSL2585_REG_H

#define TSL2585_REG_P2RAM_OTP_8                     0x08
#define TSL2585_REG_MOD_CHANNEL_CTRL                0x40
#define TSL2585_REG_ENABLE                          0x80
#define TSL2585_REG_MEAS_MODE0                      0x81
#define TSL2585_REG_MEAS_MODE1                      0x82
#define TSL2585_REG_SAMPLE_TIME0                    0x83
#define TSL2585_REG_SAMPLE_TIME1                    0x84
#define TSL2585_REG_ALS_NR_SAMPLES0                 0x85
#define TSL2585_REG_ALS_NR_SAMPLES1                 0x86
#define TSL2585_REG_FD_NR_SAMPLES0                  0x87
#define TSL2585_REG_FD_NR_SAMPLES1                  0x88
#define TSL2585_REG_WTIME                           0x89
#define TSL2585_REG_AILT0                           0x8A
#define TSL2585_REG_AILT1                           0x8B
#define TSL2585_REG_AILT2                           0x8C
#define TSL2585_REG_AIHT0                           0x8D
#define TSL2585_REG_AIHT1                           0x8E
#define TSL2585_REG_AIHT2                           0x8F
#define TSL2585_REG_AUXID                           0x90
#define TSL2585_REG_REVID                           0x91
#define TSL2585_REG_ID                              0x92
#define TSL2585_REG_STATUS                          0x93
#define TSL2585_REG_ALS_STATUS                      0x94
#define TSL2585_REG_DATAL0                          0x95
#define TSL2585_REG_DATAH0                          0x96
#define TSL2585_REG_DATAL1                          0x97
#define TSL2585_REG_DATAH1                          0x98
#define TSL2585_REG_DATAL2                          0x99
#define TSL2585_REG_DATAH2                          0x9A
#define TSL2585_REG_ALS_STATUS2                     0x9B
#define TSL2585_REG_ALS_STATUS3                     0x9C
#define TSL2585_REG_STATUS2                         0x9D
#define TSL2585_REG_STATUS3                         0x9E
#define TSL2585_REG_STATUS4                         0x9F
#define TSL2585_REG_STATUS5                         0xA0
#define TSL2585_REG_CFG0                            0xA1
#define TSL2585_REG_CFG1                            0xA2
#define TSL2585_REG_CFG2                            0xA3
#define TSL2585_REG_CFG3                            0xA4
#define TSL2585_REG_CFG4                            0xA5
#define TSL2585_REG_CFG5                            0xA6
#define TSL2585_REG_CFG6                            0xA7
#define TSL2585_REG_CFG7                            0xA8
#define TSL2585_REG_CFG8                            0xA9
#define TSL2585_REG_CFG9                            0xAA
#define TSL2585_REG_AGC_NR_SAMPLES_LO               0xAC
#define TSL2585_REG_AGC_NR_SAMPLES_HI               0xAD
#define TSL2585_REG_TRIGGER_MODE                    0xAE
#define TSL2585_REG_CONTROL                         0xB1
#define TSL2585_REG_INTENAB                         0xBA
#define TSL2585_REG_SIEN                            0xBB
#define TSL2585_REG_MOD_COMP_CFG1                   0xCE
#define TSL2585_REG_MEAS_SEQR_FD_0                  0xCF
#define TSL2585_REG_MEAS_SEQR_ALS_FD_1              0xD0
#define TSL2585_REG_MEAS_SEQR_APERS_AND_VSYNC_WAIT  0xD1
#define TSL2585_REG_MEAS_SEQR_RESIDUAL_0            0xD2
#define TSL2585_REG_MEAS_SEQR_RESIDUAL_1_AND_WAIT   0xD3
#define TSL2585_REG_MEAS_SEQR_STEP0_MOD_GAINX_L     0xD4
#define TSL2585_REG_MEAS_SEQR_STEP1_MOD_GAINX_L     0xD6
#define TSL2585_REG_MEAS_SEQR_STEP2_MOD_GAINX_L     0xD8
#define TSL2585_REG_MEAS_SEQR_STEP3_MOD_GAINX_L     0xDA
#define TSL2585_REG_MEAS_SEQR_STEP0_MOD_PHDX_SMUX_L 0xDC
#define TSL2585_REG_MEAS_SEQR_STEP0_MOD_PHDX_SMUX_H 0xDD
#define TSL2585_REG_MEAS_SEQR_STEP1_MOD_PHDX_SMUX_L 0xDE
#define TSL2585_REG_MEAS_SEQR_STEP1_MOD_PHDX_SMUX_H 0xDF
#define TSL2585_REG_MEAS_SEQR_STEP2_MOD_PHDX_SMUX_L 0xE0
#define TSL2585_REG_MEAS_SEQR_STEP2_MOD_PHDX_SMUX_H 0xE1
#define TSL2585_REG_MEAS_SEQR_STEP3_MOD_PHDX_SMUX_L 0xE2
#define TSL2585_REG_MEAS_SEQR_STEP3_MOD_PHDX_SMUX_H 0xE3
#define TSL2585_REG_MOD_CALIB_CFG0                  0xE4
#define TSL2585_REG_MOD_CALIB_CFG2                  0xE6
#define TSL2585_REG_VSYNC_PERIOD_L                  0xF2
#define TSL2585_REG_VSYNC_PERIOD_H                  0xF3
#define TSL2585_REG_VSYNC_PERIOD_TARGET_L           0xF4
#define TSL2585_REG_VSYNC_PERIOD_TARGET_H           0xF5
#define TSL2585_REG_VSYNC_CONTROL                   0xF6
#define TSL2585_REG_VSYNC_CFG                       0xF7
#define TSL2585_REG_VSYNC_GPIO_INT                  0xF8
#define TSL2585_REG_MOD_FIFO_DATA_CFG0              0xF9
#define TSL2585_REG_MOD_FIFO_DATA_CFG1              0xFA
#define TSL2585_REG_MOD_FIFO_DATA_CFG2              0xFB
#define TSL2585_REG_FIFO_THR                        0xFC
#define TSL2585_REG_FIFO_LEVEL                      0xFD
#define TSL2585_REG_FIFO_STATUS0                    0xFE
#define TSL2585_REG_FIFO_DATA                       0xFF

/* Mask Macros */
#define TSL2585_PON_SHIFT 0
#define TSL2585_MASK_PON (1 << TSL2585_PON_SHIFT)

#define TSL2585_AEN_SHIFT 1
#define TSL2585_MASK_AEN (1 << TSL2585_AEN_SHIFT)

#define TSL2585_FDEN_SHIFT 6
#define TSL2585_MASK_FDEN (1 << TSL2585_FDEN_SHIFT)

#define TSL2585_MINT_SHIFT 7
#define TSL2585_MASK_MINT (1 << TSL2585_MINT_SHIFT)

#define TSL2585_AINT_SHIFT 3
#define TSL2585_MASK_AINT (1 << TSL2585_AINT_SHIFT)

#define TSL2585_FINT_SHIFT 2
#define TSL2585_MASK_FINT (1 << TSL2585_FINT_SHIFT)

#define TSL2585_SINT_SHIFT 0
#define TSL2585_MASK_SINT (1 << TSL2585_SINT_SHIFT)

#define TSL2585_MIEN_SHIFT 7
#define TSL2585_MASK_MIEN (1 << TSL2585_MIEN_SHIFT)

#define TSL2585_AIEN_SHIFT 3
#define TSL2585_MASK_AIEN (1 << TSL2585_AIEN_SHIFT)

#define TSL2585_FIEN_SHIFT 2
#define TSL2585_MASK_FIEN (1 << TSL2585_FIEN_SHIFT)

#define TSL2585_SIEN_SHIFT 0
#define TSL2585_MASK_SIEN (1 << TSL2585_SIEN_SHIFT)

#define TSL2585_APERS_SHIFT 0
#define TSL2585_MASK_APERS (0xF << TSL2585_APERS_SHIFT)

#define TSL2585_ASAT_SHIFT 7
#define TSL2585_MASK_ASAT (1 << TSL2585_ASAT_SHIFT)

#define TSL2585_MOD_CALIB_NTH_ITERATION_AGC_ENABLE_SHIFT 5
#define TSL2585_MASK_MOD_CALIB_NTH_ITERATION_AGC_ENABLE (1 << TSL2585_MOD_CALIB_NTH_ITERATION_AGC_ENABLE_SHIFT)

#define TSL2585_MEASUREMENT_SEQUENCER_AGC_ASAT_PATTERN_SHIFT 4
#define TSL2585_MASK_MEASUREMENT_SEQUENCER_AGC_ASAT_PATTERN (0x0F << TSL2585_MEASUREMENT_SEQUENCER_AGC_ASAT_PATTERN_SHIFT)

#define TSL2585_MEASUREMENT_SEQUENCER_AGC_PREDICT_PATTERN_SHIFT 4
#define TSL2585_MASK_MEASUREMENT_SEQUENCER_AGC_PREDICT_PATTERN (0x0F << TSL2585_MEASUREMENT_SEQUENCER_AGC_PREDICT_PATTERN_SHIFT)

#define TSL2585_FIFO_OV_SHIFT 7
#define TSL2585_MASK_FIFO_OV (1 << TSL2585_FIFO_OV_SHIFT)

#define TSL2585_FIFO_UF_SHIFT 6
#define TSL2585_MASK_FIFO_UF (1 << TSL2585_FIFO_UF_SHIFT)

#define TSL2585_MOD_ALS_FIFO_DATAX_WRITE_ENABLE_SHIFT 7
#define TSL2585_MASK_MOD_ALS_FIFO_DATAX_WRITE_ENABLE (0x01 << TSL2585_MOD_ALS_FIFO_DATAX_WRITE_ENABLE_SHIFT)

#define TSL2585_FIFO_THR_LSB_SHIFT 0
#define TSL2585_MASK_FIFO_THR_LSB (0x0001 << TSL2585_FIFO_THR_LSB_SHIFT)

#define TSL2585_ALS_DATA0_SCALED_STATUS_SHIFT 2
#define TSL2585_MASK_ALS_DATA0_SCALED_STATUS (0x01 << TSL2585_ALS_DATA0_SCALED_STATUS_SHIFT)

#define TSL2585_ALS_DATA1_SCALED_STATUS_SHIFT 1
#define TSL2585_MASK_ALS_DATA1_SCALED_STATUS (0x01 << TSL2585_ALS_DATA1_SCALED_STATUS_SHIFT)

#define TSL2585_ALS_DATA2_SCALED_STATUS_SHIFT 0
#define TSL2585_MASK_ALS_DATA2_SCALED_STATUS (0x01 << TSL2585_ALS_DATA2_SCALED_STATUS_SHIFT)

#define TSL2585_FD_NR_SAMPLES_HI_SHIFT 0
#define TSL2585_MASK_FD_NR_SAMPLES_HI (0x07 << TSL2585_FD_NR_SAMPLES_HI_SHIFT)

#define TSL2585_FIFO_CLR_SHIFT 1
#define TSL2585_MASK_FIFO_CLR (0x01 << TSL2585_FIFO_CLR_SHIFT)

#define TSL2585_MOD_CALIB_NTH_ITERATION_SHIFT 0
#define TSL2585_MASK_MOD_CALIB_NTH_ITERATION (0xFF << TSL2585_MOD_CALIB_NTH_ITERATION_SHIFT)

#define TSL2585_SINT_MEASUREMENT_SEQUENCER_SHIFT 1
#define TSL2585_MASK_SINT_MEASUREMENT_SEQUENCER (0x01 << TSL2585_SINT_MEASUREMENT_SEQUENCER_SHIFT)

#define TSL2585_SAI_ACTIVE_SHIFT 1
#define TSL2585_MASK_SAI_ACTIVE (0x01 << TSL2585_SAI_ACTIVE_SHIFT)

#define TSL2585_CLEAR_SAI_ACTIVE_SHIFT 0
#define TSL2585_MASK_CLEAR_SAI_ACTIVE (0x01 << TSL2585_CLEAR_SAI_ACTIVE_SHIFT)

#define TSL2585_SAI_SHIFT 6
#define TSL2585_MASK_SAI (0x01 << TSL2585_SAI_SHIFT)

#define TSL2585_SIEN_MEASUREMENT_SEQUENCER_SHIFT 1
#define TSL2585_MASK_SIEN_MEASUREMENT_SEQUENCER (0x01 << TSL2585_SIEN_MEASUREMENT_SEQUENCER_SHIFT)

#define TSL2585_MEASUREMENT_SEQUENCER_SINGLE_SHOT_SHIFT 5
#define TSL2585_MASK_MEASUREMENT_SEQUENCER_SINGLE_SHOT (0x01 << TSL2585_MEASUREMENT_SEQUENCER_SINGLE_SHOT_SHIFT)

#define TSL2585_MEASUREMENT_SEQUENCER_MOD0_FD_PATTERN_SHIFT 0
#define TSL2585_MASK_MEASUREMENT_SEQUENCER_MOD0_FD_PATTERN (0x0F << TSL2585_MEASUREMENT_SEQUENCER_MOD0_FD_PATTERN_SHIFT)

#define TSL2585_MEASUREMENT_SEQUENCER_MOD1_FD_PATTERN_SHIFT 4
#define TSL2585_MASK_MEASUREMENT_SEQUENCER_MOD1_FD_PATTERN (0x0F << TSL2585_MEASUREMENT_SEQUENCER_MOD1_FD_PATTERN_SHIFT)

#define TSL2585_MEASUREMENT_SEQUENCER_MOD2_FD_PATTERN_SHIFT 4
#define TSL2585_MASK_MEASUREMENT_SEQUENCER_MOD2_FD_PATTERN (0x0F << TSL2585_MEASUREMENT_SEQUENCER_MOD2_FD_PATTERN_SHIFT)

#define TSL2585_MEASUREMENT_SEQUENCER_ALS_PATTERN_SHIFT 0
#define TSL2585_MASK_MEASUREMENT_SEQUENCER_ALS_PATTERN (0x0F << TSL2585_MEASUREMENT_SEQUENCER_ALS_PATTERN_SHIFT)

#endif /* TSL2585_REG_H */
